@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2009 Linpro AS
+ * Copyright (c) 2006-2011 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,21 +26,18 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * const char **ParseArgv(const char *s, int comment)
+ * const char **VAV_Parse(const char *s, int comment)
  *	Parse a command like line into an argv[]
  *	Index zero contains NULL or an error message
  *	"double quotes" and backslash substitution is handled.
  *
- * void FreeArgv(const char **argv)
- *	Free the result of ParseArgv()
+ * void VAV_Free(const char **argv)
+ *	Free the result of VAV_Parse()
  *
  */
 
 #include "config.h"
 
-#include "svnid.h"
-SVNID("$Id: argv.c 4234 2009-09-08 18:19:52Z phk $")
-
 #include <ctype.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -49,7 +46,7 @@ SVNID("$Id: argv.c 4234 2009-09-08 18:19:52Z phk $")
 #include "libvarnish.h"
 
 int
-BackSlash(const char *s, char *res)
+VAV_BackSlash(const char *s, char *res)
 {
 	int r;
 	char c;
@@ -105,7 +102,7 @@ BackSlash(const char *s, char *res)
 }
 
 char *
-BackSlashDecode(const char *s, const char *e)
+VAV_BackSlashDecode(const char *s, const char *e)
 {
 	const char *q;
 	char *p, *r;
@@ -122,7 +119,7 @@ BackSlashDecode(const char *s, const char *e)
 			*r++ = *q++;
 			continue;
 		}
-		i = BackSlash(q, r);
+		i = VAV_BackSlash(q, r);
 		q += i;
 		r++;
 	}
@@ -134,7 +131,7 @@ static char err_invalid_backslash[] = "Invalid backslash sequence";
 static char err_missing_quote[] = "Missing '\"'";
 
 char **
-ParseArgv(const char *s, int flag)
+VAV_Parse(const char *s, int *argc, int flag)
 {
 	char **argv;
 	const char *p;
@@ -157,7 +154,7 @@ ParseArgv(const char *s, int flag)
 		}
 		if ((flag & ARGV_COMMENT) && *s == '#')
 			break;
-		if (*s == '"') {
+		if (*s == '"' && !(flag & ARGV_NOESC)) {
 			p = ++s;
 			quote = 1;
 		} else {
@@ -165,8 +162,8 @@ ParseArgv(const char *s, int flag)
 			quote = 0;
 		}
 		while (1) {
-			if (*s == '\\') {
-				i = BackSlash(s, NULL);
+			if (*s == '\\' && !(flag & ARGV_NOESC)) {
+				i = VAV_BackSlash(s, NULL);
 				if (i == 0) {
 					argv[0] = err_invalid_backslash;
 					return (argv);
@@ -182,7 +179,7 @@ ParseArgv(const char *s, int flag)
 				s++;
 				continue;
 			}
-			if (*s == '"')
+			if (*s == '"' && !(flag & ARGV_NOESC))
 				break;
 			if (*s == '\0') {
 				argv[0] = err_missing_quote;
@@ -194,16 +191,26 @@ ParseArgv(const char *s, int flag)
 			argv = realloc(argv, sizeof (*argv) * (largv += largv));
 			assert(argv != NULL);
 		}
-		argv[nargv++] = BackSlashDecode(p, s);
+		if (flag & ARGV_NOESC) {
+			argv[nargv] = malloc(1 + (s - p));
+			assert(argv[nargv] != NULL);
+			memcpy(argv[nargv], p, s - p);
+			argv[nargv][s - p] = '\0';
+			nargv++;
+		} else {
+			argv[nargv++] = VAV_BackSlashDecode(p, s);
+		}
 		if (*s != '\0')
 			s++;
 	}
-	argv[nargv++] = NULL;
+	argv[nargv] = NULL;
+	if (argc != NULL)
+		*argc = nargv;
 	return (argv);
 }
 
 void
-FreeArgv(char **argv)
+VAV_Free(char **argv)
 {
 	int i;
 
@@ -217,7 +224,7 @@ FreeArgv(char **argv)
 #include <printf.h>
 
 static void
-PrintArgv(char **argv)
+VAV_Print(char **argv)
 {
 	int i;
 
@@ -234,8 +241,8 @@ Test(const char *str)
 	char **av;
 
 	printf("Test: <%V>\n", str);
-	av = ParseArgv(str, 0);
-	PrintArgv(av);
+	av = VAV_Parse(str, 0);
+	VAV_Print(av);
 }
 
 int
